@@ -21,20 +21,24 @@
 #include "filechooser.h"
 #include "utils.h"
 
+#include <QDialogButtonBox>
 #include <QDBusMetaType>
 #include <QDBusArgument>
 #include <QLoggingCategory>
-#include <QFileDialog>
-#include <QRegularExpression>
+#include <QPushButton>
+#include <QVBoxLayout>
+#include <QUrl>
+
 #include <KLocalizedString>
+#include <KFileWidget>
 
 Q_LOGGING_CATEGORY(XdgDesktopPortalKdeFileChooser, "xdp-kde-file-chooser")
 
 // Keep in sync with qflatpakfiledialog from flatpak-platform-plugin
-Q_DECLARE_METATYPE(FileChooserPortal::Filter);
-Q_DECLARE_METATYPE(FileChooserPortal::Filters);
-Q_DECLARE_METATYPE(FileChooserPortal::FilterList);
-Q_DECLARE_METATYPE(FileChooserPortal::FilterListList);
+Q_DECLARE_METATYPE(FileChooserPortal::Filter)
+Q_DECLARE_METATYPE(FileChooserPortal::Filters)
+Q_DECLARE_METATYPE(FileChooserPortal::FilterList)
+Q_DECLARE_METATYPE(FileChooserPortal::FilterListList)
 
 QDBusArgument &operator << (QDBusArgument &arg, const FileChooserPortal::Filter &filter)
 {
@@ -78,35 +82,26 @@ const QDBusArgument &operator >> (const QDBusArgument &arg, FileChooserPortal::F
     return arg;
 }
 
-static bool isGtkFilterPattern(const QString &pattern)
+FileDialog::FileDialog(QDialog *parent, Qt::WindowFlags flags)
+    : QDialog(parent, flags)
+    , m_fileWidget(new KFileWidget(QUrl(), this))
 {
-    // We are looking for patterns containing regexp looking like "[Pp][Nn][Gg]"
-    QRegularExpression re(QStringLiteral("\\*\\.(\\w*(\\[\\w+\\])+\\w*)+"));
-    return re.match(pattern).hasMatch();
+    setLayout(new QVBoxLayout);
+    layout()->addWidget(m_fileWidget);
+
+    m_buttons = new QDialogButtonBox(this);
+    m_buttons->addButton(m_fileWidget->okButton(), QDialogButtonBox::AcceptRole);
+    m_buttons->addButton(m_fileWidget->cancelButton(), QDialogButtonBox::RejectRole);
+    connect(m_buttons, &QDialogButtonBox::rejected, m_fileWidget, &KFileWidget::slotCancel);
+    connect(m_fileWidget->okButton(), &QAbstractButton::clicked, m_fileWidget, &KFileWidget::slotOk);
+    connect(m_fileWidget, &KFileWidget::accepted, m_fileWidget, &KFileWidget::accept);
+    connect(m_fileWidget, &KFileWidget::accepted, this, &QDialog::accept);
+    connect(m_fileWidget->cancelButton(), &QAbstractButton::clicked, this, &QDialog::reject);
+    layout()->addWidget(m_buttons);
 }
 
-static QString gtkToQtFilterPattern(const QString &pattern)
+FileDialog::~FileDialog()
 {
-    QRegularExpression re(QStringLiteral("[\\[\\]]"));
-
-    QString result;
-    const QStringList list = pattern.split(re, QString::SkipEmptyParts);
-    for (const QString &str : list) {
-        if (str.startsWith(QStringLiteral("*."))) {
-            result += str;
-        } else {
-            // Check if first and last letter are same
-            if (str.at(0).toLower() == str.at(str.length() - 1).toLower()) {
-                // Take first letter and convert it to lower, so for example from "[Aa]" we get just "a"
-                result += str.at(0).toLower();
-            } else {
-                // If not, take whole string as it's probably a string between two patterns, like [aA]bc[dD]
-                result += str;
-            }
-        }
-    }
-
-    return result;
 }
 
 FileChooserPortal::FileChooserPortal(QObject *parent)
@@ -155,67 +150,61 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
      * Example: [('encoding', 'Encoding', [('utf8', 'Unicode (UTF-8)'), ('latin15', 'Western')], 'latin15'), ('reencode', 'Reencode', [], 'false')]
      */
 
-    if (options.contains(QLatin1String("accept_label"))) {
-        acceptLabel = options.value(QLatin1String("accept_label")).toString();
+    if (options.contains(QStringLiteral("accept_label"))) {
+        acceptLabel = options.value(QStringLiteral("accept_label")).toString();
     }
 
-    if (options.contains(QLatin1String("modal"))) {
-        modalDialog = options.value(QLatin1String("modal")).toBool();
+    if (options.contains(QStringLiteral("modal"))) {
+        modalDialog = options.value(QStringLiteral("modal")).toBool();
     }
 
-    if (options.contains(QLatin1String("multiple"))) {
-        multipleFiles = options.value(QLatin1String("multiple")).toBool();
+    if (options.contains(QStringLiteral("multiple"))) {
+        multipleFiles = options.value(QStringLiteral("multiple")).toBool();
     }
 
-    if (options.contains(QLatin1String("filters"))) {
-        FilterListList filterListList = qdbus_cast<FilterListList>(options.value(QLatin1String("filters")));
-        Q_FOREACH (const FilterList &filterList, filterListList) {
+    if (options.contains(QStringLiteral("filters"))) {
+        FilterListList filterListList = qdbus_cast<FilterListList>(options.value(QStringLiteral("filters")));
+        for (const FilterList &filterList : filterListList) {
             QStringList filterStrings;
-            Q_FOREACH (const Filter &filterStruct, filterList.filters) {
+            for (const Filter &filterStruct : filterList.filters) {
                 if (filterStruct.type == 0) {
-                    QString filterString = filterStruct.filterString;
-                    if (isGtkFilterPattern(filterString)) {
-                        filterString = gtkToQtFilterPattern(filterString);
-                    }
-                    filterStrings << filterString;
+                    filterStrings << filterStruct.filterString;
                 } else {
                     mimeTypeFilters << filterStruct.filterString;
                 }
             }
 
             if (!filterStrings.isEmpty()) {
-                nameFilters << QString("%1 (%2)").arg(filterList.userVisibleName).arg(filterStrings.join(QLatin1String(" ")));
+                nameFilters << QStringLiteral("%1|%2").arg(filterStrings.join(QLatin1Char(' '))).arg(filterList.userVisibleName);
             }
         }
     }
 
-    QFileDialog *fileDialog = new QFileDialog();
-    Utils::setParentWindow(fileDialog, parent_window);
+    QScopedPointer<FileDialog, QScopedPointerDeleteLater> fileDialog(new FileDialog());
+    Utils::setParentWindow(fileDialog.data(), parent_window);
     fileDialog->setWindowTitle(title);
     fileDialog->setModal(modalDialog);
-    fileDialog->setFileMode(multipleFiles ? QFileDialog::ExistingFiles : QFileDialog::ExistingFile);
-    fileDialog->setLabelText(QFileDialog::Accept, !acceptLabel.isEmpty() ? acceptLabel : i18n("Open"));
+    fileDialog->m_fileWidget->setMode(multipleFiles ? KFile::Mode::File | KFile::Mode::ExistingOnly : KFile::Mode::Files | KFile::Mode::ExistingOnly);
+    fileDialog->m_fileWidget->okButton()->setText(!acceptLabel.isEmpty() ? acceptLabel : i18n("Open"));
 
     if (!nameFilters.isEmpty()) {
-        fileDialog->setNameFilters(nameFilters);
+        fileDialog->m_fileWidget->setFilter(nameFilters.join(QLatin1Char('\n')));
     }
 
     if (!mimeTypeFilters.isEmpty()) {
-        fileDialog->setMimeTypeFilters(mimeTypeFilters);
+        fileDialog->m_fileWidget->setMimeFilter(mimeTypeFilters);
     }
 
     if (fileDialog->exec() == QDialog::Accepted) {
         QStringList files;
-        Q_FOREACH (const QString &filename, fileDialog->selectedFiles()) {
+        for (const QString &filename : fileDialog->m_fileWidget->selectedFiles()) {
            QUrl url = QUrl::fromLocalFile(filename);
            files << url.toDisplayString();
         }
-        results.insert(QLatin1String("uris"), files);
-        fileDialog->deleteLater();
+        results.insert(QStringLiteral("uris"), files);
         return 0;
     }
 
-    fileDialog->deleteLater();
     return 1;
 }
 
@@ -244,90 +233,83 @@ uint FileChooserPortal::SaveFile(const QDBusObjectPath &handle,
 
     // TODO parse options - choices
 
-    if (options.contains(QLatin1String("modal"))) {
-        modalDialog = options.value(QLatin1String("modal")).toBool();
+    if (options.contains(QStringLiteral("modal"))) {
+        modalDialog = options.value(QStringLiteral("modal")).toBool();
     }
 
-    if (options.contains(QLatin1String("accept_label"))) {
-        acceptLabel = options.value(QLatin1String("accept_label")).toString();
+    if (options.contains(QStringLiteral("accept_label"))) {
+        acceptLabel = options.value(QStringLiteral("accept_label")).toString();
     }
 
-    if (options.contains(QLatin1String("current_name"))) {
-        currentName = options.value(QLatin1String("current_name")).toString();
+    if (options.contains(QStringLiteral("current_name"))) {
+        currentName = options.value(QStringLiteral("current_name")).toString();
     }
 
-    if (options.contains(QLatin1String("current_folder"))) {
-        currentFolder = options.value(QLatin1String("current_folder")).toByteArray();
+    if (options.contains(QStringLiteral("current_folder"))) {
+        currentFolder = QString::fromUtf8(options.value(QStringLiteral("current_folder")).toByteArray());
     }
 
-    if (options.contains(QLatin1String("current_file"))) {
-        currentFile = options.value(QLatin1String("current_file")).toByteArray();
+    if (options.contains(QStringLiteral("current_file"))) {
+        currentFile = QString::fromUtf8(options.value(QStringLiteral("current_file")).toByteArray());
     }
 
-    if (options.contains(QLatin1String("filters"))) {
-        FilterListList filterListList = qdbus_cast<FilterListList>(options.value(QLatin1String("filters")));
-        Q_FOREACH (const FilterList &filterList, filterListList) {
+    if (options.contains(QStringLiteral("filters"))) {
+        FilterListList filterListList = qdbus_cast<FilterListList>(options.value(QStringLiteral("filters")));
+        for (const FilterList &filterList : filterListList) {
             QStringList filterStrings;
-            Q_FOREACH (const Filter &filterStruct, filterList.filters) {
+            for (const Filter &filterStruct : filterList.filters) {
                 if (filterStruct.type == 0) {
-                    QString filterString = filterStruct.filterString;
-                    if (isGtkFilterPattern(filterString)) {
-                        filterString = gtkToQtFilterPattern(filterString);
-                    }
-                    filterStrings << filterString;
+                    filterStrings << filterStruct.filterString;
                 } else {
                     mimeTypeFilters << filterStruct.filterString;
                 }
             }
 
             if (!filterStrings.isEmpty()) {
-                nameFilters << QString("%1 (%2)").arg(filterList.userVisibleName).arg(filterStrings.join(QLatin1String(" ")));
+                nameFilters << QStringLiteral("%1|%2").arg(filterStrings.join(QLatin1Char(' '))).arg(filterList.userVisibleName);
             }
         }
     }
 
-    QFileDialog *fileDialog = new QFileDialog();
-    Utils::setParentWindow(fileDialog, parent_window);
+    QScopedPointer<FileDialog, QScopedPointerDeleteLater> fileDialog(new FileDialog());
+    Utils::setParentWindow(fileDialog.data(), parent_window);
     fileDialog->setWindowTitle(title);
     fileDialog->setModal(modalDialog);
-    fileDialog->setAcceptMode(QFileDialog::AcceptSave);
+    fileDialog->m_fileWidget->setOperationMode(KFileWidget::Saving);
 
     if (!currentFolder.isEmpty()) {
-        fileDialog->setDirectoryUrl(QUrl(currentFolder));
+        fileDialog->m_fileWidget->setUrl(QUrl::fromLocalFile(currentFolder));
     }
 
     if (!currentFile.isEmpty()) {
-        fileDialog->selectFile(currentFile);
+        fileDialog->m_fileWidget->setSelectedUrl(QUrl::fromLocalFile(currentFile));
     }
 
     if (!currentName.isEmpty()) {
-        fileDialog->selectFile(currentName);
+        const QUrl url = fileDialog->m_fileWidget->baseUrl();
+        fileDialog->m_fileWidget->setSelectedUrl(QUrl::fromLocalFile(QStringLiteral("%1/%2").arg(url.toDisplayString(QUrl::StripTrailingSlash), currentName)));
     }
 
     if (!acceptLabel.isEmpty()) {
-        fileDialog->setLabelText(QFileDialog::Accept, acceptLabel);
+        fileDialog->m_fileWidget->okButton()->setText(acceptLabel);
     }
 
     if (!nameFilters.isEmpty()) {
-        fileDialog->setNameFilters(nameFilters);
+        fileDialog->m_fileWidget->setFilter(nameFilters.join(QLatin1Char('\n')));
     }
 
     if (!mimeTypeFilters.isEmpty()) {
-        fileDialog->setMimeTypeFilters(mimeTypeFilters);
+        fileDialog->m_fileWidget->setMimeFilter(mimeTypeFilters);
     }
 
     if (fileDialog->exec() == QDialog::Accepted) {
         QStringList files;
-        Q_FOREACH (const QString &filename, fileDialog->selectedFiles()) {
-           QUrl url = QUrl::fromLocalFile(filename);
-           files << url.toDisplayString();
-        }
-        results.insert(QLatin1String("uris"), files);
-        fileDialog->deleteLater();
+        QUrl url = QUrl::fromLocalFile(fileDialog->m_fileWidget->selectedFile());
+        files << url.toDisplayString();
+        results.insert(QStringLiteral("uris"), files);
         return 0;
     }
 
-    fileDialog->deleteLater();
     return 1;
 }
 
